<?php
/* $Id: eventgui.class.php 132 2011-03-09 20:42:39Z mir $ */

include_once 'config.inc.php';
include_once 'dateformat.class.php';
include_once 'caldavresource.class.php';
require_once 'calendar.class.php';
require_once 'helper.php';

class EventGUI {

    private $event;
    private $referer;
    private $summary;
    private $location;
    private $description;
    private $start;
    private $end;
    private $timezone;
    private $dates;
    private $rrule;
    private $startdate;
    private $enddate;
    private $until;
    private $cal;

    function __construct(VEvent $event = NULL, $referer = '', $cal = NULL) {
        $this->event = $event;
        $this->referer = urldecode($referer);
        $this->cal = $cal;
        //print $this->referer;
        //exit;
        $this->summary = '';
        $this->location = '';
        $this->description = '';
        $this->start = '';
        $this->end = '';
        $this->timezone = '';
        $this->localtime = ($_SESSION['user_settings']->getTimeZone() !== NULL) ?
            $_SESSION['user_settings']->getTimeZone() : TIMEZONE;
        $root = WEB_ROOT;
        $week_start_sunday = ($_SESSION['user_settings']->getStartWeek() !== NULL) ?
            $_SESSION['user_settings']->getStartWeek() : WEEK_START_SUNDAY;
        $week_start_sunday = ($week_start_sunday) ? 0 : 1;
        $this->startdate = <<< __CAL1
    <script type="text/javascript">
        A_TCALDEF['imgpath'] = '{$root}js/img/';
        A_TCALDEF['weekstart'] = $week_start_sunday;
        new tcal ({
        'formname': 'event_data',
        'controlname': 'startdate'
    });
    </script>
__CAL1;
        $this->enddate = <<< __CAL1
    <script type="text/javascript">
        A_TCALDEF['imgpath'] = '{$root}js/img/';
        A_TCALDEF['weekstart'] = $week_start_sunday;
        new tcal ({
        'formname': 'event_data',
        'controlname': 'enddate'
    });
    </script>
__CAL1;
        $this->until = <<< __CAL1
    <script type="text/javascript">
        A_TCALDEF['imgpath'] = '{$root}js/img/';
        A_TCALDEF['weekstart'] = $week_start_sunday;
        new tcal ({
        'formname': 'event_data',
        'controlname': 'untildate'
    });
    </script>
__CAL1;
    }

    private function parseEvent() {
        if (! $this->event)
            return FALSE;
        $comp = $this->event->getBaseComponent();
        $this->summary = $comp->GetPValue('SUMMARY');
        $this->location = $comp->GetPValue('LOCATION');
        $this->description = $comp->GetPValue('DESCRIPTION');
        $this->start = $comp->GetPValue('DTSTART');
        $this->end = $comp->GetPValue('DTEND');
        $this->timezone = $this->event->getTZID();
        $rrule = $this->event->getRRule();
        $this->dates = $rrule->getEventDates();
        $this->rrule = $rrule->getAll();
        //echo $this->start .':'. $this->end .'<br/>';
        return TRUE;
    }

    private function createDT($elem, $name) {
        $res = array();

        //print_r($elem);
        //echo '<br/>';
        if (count($elem) > 1) {
            $date = $elem[0];
            $time = substr($elem[1], 0, 4);
        }
        else {
            $date = $elem[0];
            $time = '2359';
        }
        for ($i = 0; $i < substr($time, 2); $i += 15);
        if ($i < 10)
            $i = "0$i";
        $time = substr($time, 0, 2) . $i;
        //echo "$time<br/>";
        $options = array();
        for ($i = 0; $i < 24; $i++) {
            $h = ($i < 10) ? "0$i" : $i;
            $stop = ($i == 23) ? 61 : 60;
            for ($j = 0; $j < $stop; $j += 15) {
                $m = ($j < 10) ? "0$j" : $j;
                if ($m == 60)
                    $m = 59;
                $hm = "$h$m";
                //echo "$hm<br/>";
                $option = "<option value=\"$hm\"";
                if ($hm == $time)
                    $option .= ' selected="selected"';
                $option .= ">$hm</option>";
                array_push($options, $option);
            }
        }
        $select = "<select name=\"$name\" id=\"$name\">";
        foreach ($options as $option)
            $select .= $option;
        $select .= '</select>';
        $date = substr($date, 0, 4).'-'.substr($date, 4, 2).'-'.substr($date, 6);
        //echo "$date<br/>";
        $res['date'] = $date;
        $res['time'] = $select;
        return $res;
    }

    private function createTime() {
        include_once 'timezone.php';

        if ($this->start == '' || $this->end == '') {
            $this->start = date("Ymd\THis");
            $this->end = date("Ymd\THis", strtotime("+1 hour"));
            $this->timezone = $_SESSION['user_settings']->getTimeZone();
        }
        //echo $this->start .':'. $this->end .'<br/>';
        $tz = '<select name="tzid">';
        foreach ($timezones as $timezone) {
            $tz .= "<option value=\"$timezone\"";
            if ($this->localtime == $timezone)
                $tz .= ' selected="selected"';
            $tz .= ">$timezone</option>";
        }
        $tz .= "</select>";
        $dateFormat = new DateFormat();
        $UTCStart = $dateFormat->UTC2Client($this->start);
        $UTCEnd = $dateFormat->UTC2Client($this->end);
        //echo $UTCStart .':'. $UTCEnd .'<br/>';
        $startArray = explode('T', $UTCStart);
        //echo "Client: ".$dateFormat->client2UTC($dateFormat->UTC2Client($this->start))."<br/>";
        //echo "Local: ".$dateFormat->UTC2Local($this->start)."<br/>";
        $start = $this->createDT($startArray, 'starttime');
        $endArray = explode('T', $UTCEnd);
        $end = $this->createDT($endArray, 'endtime');
        $allDay = CaldavRessource::allDayEvent($UTCStart, $UTCEnd);
        $allDay = ($allDay) ? 'checked="checked"' : '';
        $datetime = "<table><caption style=\"font-weight: bold\">Date &amp; Time</caption>
        <tr>
            <td>All Day Event</td>
            <td>
            <script type=\"text/javascript\">
                var observable1 = ['starttime', 'tcalico_1', 'enddate', 'endtime'];
                document.write('<input type=\"checkbox\" name=\"allDay\" $allDay' +
                        'id=\"allDay\" onchange=\"stateSwitch(this, observable1, 0)\"/>');
            </script>
            </td>
            <td>&nbsp;</td>
        </tr>
        <tr>
            <td>Start</td>
            <td>
            <script type=\"text/javascript\">
                document.write('<input type=\"text\" name=\"startdate\" ' +
                'id=\"startdate\" size=\"12\" onchange=\"dateAlign(' +
                'this)\" value=\"{$start['date']}\"/>');
            </script>
                $this->startdate
                <!--{$start['date']}-->
            </td>
            <td>{$start['time']}</td>
            <td rowspan=\"2\">$tz</td>
        </tr>
        <tr>
            <td>End</td>
            <td>
            <script type=\"text/javascript\">
                document.write('<input name=\"enddate\" type=\"text\" ' +
                'id=\"enddate\" size=\"12\" onchange=\"dateAlign(' +
                'this)\" value=\"{$end['date']}\"/>');
            </script>
                $this->enddate
                <!--{$end['date']}-->
            </td>
            <td>
                {$end['time']}
                <script type=\"text/javascript\">
                    var observable4 = ['starttime', 'tcalico_1', 'enddate', 'endtime'];
                    stateSwitch(document.getElementById('allDay'), observable4, 1);
                </script>
            </td>
        </tr>
        </table>";
        return $datetime;
    }

    private function recurRule($checked) {
        $choices = array('DAILY', 'WEEKLY', 'MONTHLY', 'YEARLY');
        $rule = '<span style="font-weight: bold">Recurrence Types</span><br/>';
        foreach ($choices as $choice) {
            $rule .= "<input type=\"radio\" name=\"recurrence\" value=\"$choice\"";
            if (strtoupper($checked) == $choice)
                $rule .= ' checked="checked"';
            $rule .= "/>$choice<br/>";
        }
        return $rule;
    }

    private function rangeRule($rules) {
        $until = '';
        $times = '';
        $count_check = '';
        $until_check = '';
        $no_check = '';

        $range = $rules['until'][0];
        if (! $range) {
            $range = $rules['count'][0];
            if ($range) {
                $times = $range;
                $count_check = 'checked="checked"';
            }
            else {
                $no_check = 'checked="checked"';
            }
        }
        else {
            $until = substr($range, 0, 4).'-'.substr($range, 4, 2).
                '-'.substr($range, 6, 2);
            $until_check = 'checked="checked"';
        }
        $rule = <<<_RULE
        <span style="font-weight: bold">Recurrence Range</span><br/>
        <input type="radio" name="range" value="UNLIMITED" $no_check/>
            No ending date<br/>
        <input type="radio" name="range" value="COUNT" $count_check/>
            End after
        <input type="text" name="times" size="4"
            value="$times"/>&nbsp;Occurrence(s)<br/>
        <input type="radio" name="range" value="UNTIL" $until_check/>
            End on
        <input type="text" name="untildate" size="12" value="$until"/>
            {$this->until}
_RULE;
        return $rule;
    }

    private function createRecur() {
        $recur = NULL;
        $rrule = NULL;
        if ($this->rrule) {
            $rrule = array_diff_key($this->rrule, array('freq' => ''));
            switch ($this->rrule['freq'][0]) {
                case 'DAILY': $recur = 'DAILY'; break;
                case 'WEEKLY': $recur = 'WEEKLY'; break;
                case 'MONTHLY': $recur = 'MONTHLY'; break;
                case 'YEARLY': $recur = 'YEARLY'; break;
            }
        }
        $recur = $this->recurRule($recur);
        $range = $this->rangeRule($rrule);
        //print htmlentities($recur);
        $res = <<<_TABLE
        <table cellpadding="10">
            <caption style="font-weight: bold">Recurrence Rule</caption>
            <tr>
                <td valign="top">$recur</td>
                <td valign="top">$range</td>
            </tr>
        </table>
_TABLE;
        return $res;
    }

    function getView() {
        $title = '<input name="summary" size="53" value="_SUM_"/>';
        $location = '<input name="location" size="53" value="_LOC_"/>';
        $description =
            '<textarea name="description" cols="60" rows="5">_DESC_</textarea>';
        if ($this->parseEvent()) {
            $title = str_replace('_SUM_', $this->summary, $title);
            $location = str_replace('_LOC_', $this->location, $location);
            $description = str_replace('_DESC_', $this->description, $description);
        }
        else {
            $title = str_replace('_SUM_', '', $title);
            $location = str_replace('_LOC_', '', $location);
            $description = str_replace('_DESC_', '', $description);
        }
        $btn_label = ($_SESSION['EVENT_ACTION'] == 'NEW') ? "Create" : "Update";
        unset($_SESSION['EVENT_ACTION']);
        // for delete button
        $etag = $this->event->getEtag();
        $url = $this->event->getUrl();
        $view = dirname($_SERVER['PHP_SELF']).'/delete_event.php';
        //['starttime', 'enddate', 'endtime']
        $view = "<div id=\"event\">
        <form id=\"event_data\" method=\"post\" action=\"{$_SERVER['PHP_SELF']}\" 
            onsubmit=\"return enableAll(['starttime', 'enddate', 'endtime'])\">
            <table>
                <caption style=\"font-weight: bold\">General</caption>
               <tr>
                   <td>Title</td><td>$title</td>
               </tr>
               <tr>
                    <td>Location</td><td>$location</td>
               </tr>
               <tr>
                    <td>Description</td><td>$description</td>
               </tr>
            </table><p/>".
            $this->createTime()."<p/>".
            $this->createRecur().
        "<p style=\"text-align: center\">
            <input type=\"hidden\" value=\"$this->referer\" name=\"referer\"/>
            <input type=\"button\" value=\"Cancel\"
                onclick=\"document.location.href='$this->referer'\"/>
            <input type=\"submit\" value=\"$btn_label\"/>
                <!--onclick=\"document.getElementById('event_data').submit()\"/>-->
            <input type=\"button\" value=\"Delete\"
                onclick=\"document.location.href='$view?etag=$etag&url=$url'+
                '&cal=$this->cal&referer=$this->referer'\"/>
            <!--onclick=\"javascript: alert('Update not available')\"/>-->
            <!--<input type=\"submit\" name=\"update\" value=\"Update\"/>-->
        </p>
        </form>
        </div>";
        return $view;
    }

}

?>
